@@ -38,7 +38,8 @@ class _ClientBase:
         Provides information about the current client. Name etc...
     """
 
-    def __init__(self, pronote_url, username: str = '', password: str = '', ent: Optional[Callable] = None):
+    def __init__(self, pronote_url: str, username: str = '', password: str = '',
+                 ent: Optional[Callable] = None) -> None:
         log.info('INIT')
         # start communication session
         if not len(password) + len(username):
@@ -66,12 +67,15 @@ class _ClientBase:
 
         self.parametres_utilisateur: dict = {}
         self.auth_cookie: dict = {}
-        self.info: Optional[dataClasses.ClientInfo] = None
+        self.info: dataClasses.ClientInfo
 
         self.start_day = datetime.datetime.strptime(
             self.func_options['donneesSec']['donnees']['General']['PremierLundi']['V'], '%d/%m/%Y').date()
         self.week = self.get_week(datetime.date.today())
 
+        self._refreshing = False
+
+        self.periods_: Optional[List[dataClasses.Period]]
         self.periods_ = self.periods
         self.logged_in = self._login()
         self._expired = False
@@ -125,9 +129,13 @@ class _ClientBase:
             e.aes_key = MD5.new((u + motdepasse).encode()).digest()
 
         # challenge
-        dec = e.aes_decrypt(bytes.fromhex(challenge))
-        dec_no_alea = _enleverAlea(dec.decode())
-        ch = e.aes_encrypt(dec_no_alea.encode()).hex()
+        try:
+            dec = e.aes_decrypt(bytes.fromhex(challenge))
+            dec_no_alea = _enleverAlea(dec.decode())
+            ch = e.aes_encrypt(dec_no_alea.encode()).hex()
+        except CryptoError as ex:
+            ex.args += "exception happened during login -> probably bad username/password",
+            raise
 
         # send
         auth_json = {"connexion": 0, "challenge": ch, "espace": int(self.attributes['a'])}
@@ -139,7 +147,7 @@ class _ClientBase:
 
             # getting listeOnglets separately because of pronote API change
             self.parametres_utilisateur = self.post('ParametresUtilisateur')
-            self.info = dataClasses.ClientInfo(self.parametres_utilisateur['donneesSec']['donnees']['ressource'])
+            self.info = dataClasses.ClientInfo(self, self.parametres_utilisateur['donneesSec']['donnees']['ressource'])
             self.communication.authorized_onglets = _prepare_onglets(
                 self.parametres_utilisateur['donneesSec']['donnees']['listeOnglets'])
             log.info("got onglets data.")
@@ -148,7 +156,7 @@ class _ClientBase:
             log.info('login failed')
             return False
 
-    def get_week(self, date: Union[datetime.date, datetime.datetime]):
+    def get_week(self, date: Union[datetime.date, datetime.datetime]) -> int:
         if isinstance(date, datetime.datetime):
             return 1 + int((date.date() - self.start_day).days / 7)
         return 1 + int((date - self.start_day).days / 7)
@@ -168,14 +176,14 @@ class _ClientBase:
         json = self.func_options['donneesSec']['donnees']['General']['ListePeriodes']
         return [dataClasses.Period(self, j) for j in json]
 
-    def keep_alive(self):
+    def keep_alive(self) -> _KeepAlive:
         """
         Returns a context manager to keep the connection alive. When inside the context manager,
         it sends a "Presence" packet to the server after 5 minutes of inactivity from another thread.
         """
         return _KeepAlive(self)
 
-    def refresh(self):
+    def refresh(self) -> None:
         """
         Now this is the true jank part of this program. It refreshes the connection if something went wrong.
         This is the classical procedure if something is broken.
@@ -209,7 +217,7 @@ class _ClientBase:
             return True
         return False
 
-    def post(self, function_name: str, onglet: int = None, data: dict = None):
+    def post(self, function_name: str, onglet: int = None, data: dict = None) -> dict:
         """
         Preforms a raw post to the PRONOTE server. Adds signature, then passes it to _Communication.post
 
@@ -237,7 +245,15 @@ class _ClientBase:
 
             log.info(
                 f'Have you tried turning it off and on again? ERROR: {e.pronote_error_code} | {e.pronote_error_msg}')
-            self.refresh()
+
+            # prevent refresh recursion
+            if self._refreshing:
+                raise e
+            else:
+                self._refreshing = True
+                self.refresh()
+                self._refreshing = False
+
             return self.communication.post(function_name, post_data)
 
 
@@ -269,7 +285,8 @@ class Client(_ClientBase):
         Provides information about the current client. Name etc...
     """
 
-    def __init__(self, pronote_url, username: str = '', password: str = '', ent: Optional[Callable] = None):
+    def __init__(self, pronote_url: str, username: str = '', password: str = '',
+                 ent: Optional[Callable] = None) -> None:
         super().__init__(pronote_url, username, password, ent)
 
     def lessons(self, date_from: Union[datetime.date, datetime.datetime],
@@ -317,6 +334,38 @@ class Client(_ClientBase):
         # since we only have week precision, we need to make it more precise on our own
         return [lesson for lesson in output if date_from <= lesson.start <= date_to]
 
+    def export_ical(self, timezone_shift: int = 0) -> str:
+        """
+        Exports ICal URL for the client's timetable
+        Parameters
+        ----------
+        timezone_shift : int
+            in what timezone should the exported calendar be in (hour shift)
+
+        Returns
+        -------
+        URL for the exported ICal file
+        """
+        user = self.parametres_utilisateur['donneesSec']['donnees']['ressource']
+        data = {
+            "ressource": user,
+            "avecAbsencesEleve": False, "avecConseilDeClasse": True,
+            "estEDTPermanence": False, "avecAbsencesRessource": True,
+            "avecDisponibilites": True, "avecInfosPrefsGrille": True,
+            "Ressource": user,
+            "NumeroSemaine": 1,
+            "numeroSemaine": 1
+        }
+        response = self.post("PageEmploiDuTemps", 16, data)
+        icalsecurise = response["donneesSec"]["donnees"].get("ParametreExportiCal")
+        if not icalsecurise:
+            raise ICalExportError("Pronote did not return ICal token")
+
+        ver = self.func_options["donneesSec"]["donnees"]["General"]["versionPN"]
+        param = f"lh={timezone_shift}".encode().hex()
+
+        return f"{self.communication.root_site}/ical/Edt.ics?icalsecurise={icalsecurise}&version={ver}&param={param}"
+
     def homework(self, date_from: datetime.date, date_to: datetime.date = None) -> List[dataClasses.Homework]:
         """
         Get homework between two given points.
@@ -393,13 +442,51 @@ class Client(_ClientBase):
 
         return info
 
+    def menus(self, date_from: datetime.date, date_to: datetime.date = None) -> List[dataClasses.Menu]:
+        """
+        Get menus between two given points.
+
+        date_from : datetime
+            The first date
+        date_to : datetime
+            The second date. If unspecified to the end of the year.
+
+        Returns
+        -------
+        List[Menu]
+            Menu between two given points
+        """
+        output = []
+
+        if not date_to:
+            date_to = date_from
+
+        first_day = date_from - datetime.timedelta(days=date_from.weekday())
+
+        # getting menus for all the weeks.
+        while first_day <= date_to:
+            data = {'date': {
+                '_T': 7,
+                'V': first_day.strftime('%d/%m/%Y') + " 0:0:0"
+            }}
+            response = self.post('PageMenus', 10, data)
+            l_list = response['donneesSec']['donnees']['ListeJours']['V']
+            for day in l_list:
+                for menu in day['ListeRepas']['V']:
+                    menu['Date'] = day['Date']
+                    output.append(dataClasses.Menu(self, menu))
+            first_day += datetime.timedelta(days=7)
+
+        # since we only have week precision, we need to make it more precise on our own
+        return [menu for menu in output if date_from <= menu.date <= date_to]
+
     @property
     def current_period(self) -> dataClasses.Period:
         """Get the current period."""
         id_period = \
             self.parametres_utilisateur['donneesSec']['donnees']['ressource']['listeOngletsPourPeriodes']['V'][0][
                 'periodeParDefaut']['V']['N']
-        return dataClasses.Util.get(self.periods_, id=id_period)[0]
+        return dataClasses.Util.get(self.periods, id=id_period)[0]
 
 
 class ParentClient(Client):
@@ -429,12 +516,13 @@ class ParentClient(Client):
         List of sub-clients representing all the children connected to the main parent account.
     """
 
-    def __init__(self, pronote_url, username: str = '', password: str = '', ent: Optional[Callable] = None):
+    def __init__(self, pronote_url: str, username: str = '', password: str = '',
+                 ent: Optional[Callable] = None) -> None:
         super().__init__(pronote_url, username, password, ent)
 
         self.children: List[dataClasses.ClientInfo] = []
         for c in self.parametres_utilisateur['donneesSec']['donnees']['ressource']['listeRessources']:
-            self.children.append(dataClasses.ClientInfo(c))
+            self.children.append(dataClasses.ClientInfo(self, c))
 
         if not self.children:
             raise ChildNotFound('No children were found.')
@@ -463,7 +551,7 @@ class ParentClient(Client):
         self._selected_child = c
         self.parametres_utilisateur['donneesSec']['donnees']['ressource'] = self._selected_child.raw_resource
 
-    def post(self, function_name: str, onglet: int = None, data: dict = None):
+    def post(self, function_name: str, onglet: int = None, data: dict = None) -> dict:
         """
         Preforms a raw post to the PRONOTE server. Adds signature, then passes it to _Communication.post
 
@@ -523,7 +611,8 @@ class VieScolaireClient(_ClientBase):
         List of all classes this account has access to.
     """
 
-    def __init__(self, pronote_url, username: str = '', password: str = '', ent: Optional[Callable] = None):
+    def __init__(self, pronote_url: str, username: str = '', password: str = '',
+                 ent: Optional[Callable] = None) -> None:
         super().__init__(pronote_url, username, password, ent)
         self.classes = [dataClasses.StudentClass(self, json) for json in
                         self.parametres_utilisateur["donneesSec"]["donnees"]["listeClasses"]["V"]]
