@@ -1,19 +1,23 @@
+from __future__ import annotations
+
 import datetime
 import json
 import logging
 import re
 from html import unescape
-from typing import Union, List, Callable, Any, TypeVar, Optional, Tuple, Set
+from typing import Union, List, Callable, Any, TypeVar, Optional, Tuple, Set, Iterable, TYPE_CHECKING
 from urllib.parse import quote
 
 from Crypto.Util import Padding
 
-from .exceptions import ParsingError
+if TYPE_CHECKING:
+    from .clients import _ClientBase
+from .exceptions import ParsingError, DateParsingError
 
 log = logging.getLogger(__name__)
 
 
-def _get_l(d): return d['L']
+def _get_l(d: dict) -> str: return d['L']
 
 
 class MissingType: pass
@@ -25,7 +29,7 @@ class Util:
                        'Felicitations']
 
     @classmethod
-    def get(cls, iterable, **kwargs) -> list:
+    def get(cls, iterable: Iterable, **kwargs: Any) -> list:
         """Gets items from the list with the attributes specified.
 
         Parameters
@@ -44,14 +48,14 @@ class Util:
         return output
 
     @classmethod
-    def grade_parse(cls, string):
+    def grade_parse(cls, string: str) -> str:
         if '|' in string:
             return cls.grade_translate[int(string[1]) - 1]
         else:
             return string
 
     @staticmethod
-    def date_parse(formatted_date: str) -> Optional[datetime.date]:
+    def date_parse(formatted_date: str) -> datetime.date:
         """convert date to a datetime.date object"""
         if re.match(r"\d{2}/\d{2}/\d{4}$", formatted_date):
             return datetime.datetime.strptime(formatted_date, '%d/%m/%Y').date()
@@ -60,11 +64,10 @@ class Util:
         elif re.match(r"\d{2}/\d{2}/\d{2} \d{2}h\d{2}$", formatted_date):
             return datetime.datetime.strptime(formatted_date, '%d/%m/%y %Hh%M').date()
         else:
-            log.warning("date parsing not possible for value '" + formatted_date + "' set to None")
-            return None
+            raise DateParsingError("Could not parse date", formatted_date)
 
     @staticmethod
-    def datetime_parse(formatted_date: str) -> Optional[datetime.datetime]:
+    def datetime_parse(formatted_date: str) -> datetime.datetime:
         """convert date to a datetime.datetime object"""
         if re.match(r"\d{2}/\d{2}/\d{4}$", formatted_date):
             return datetime.datetime.strptime(formatted_date, '%d/%m/%Y')
@@ -73,8 +76,7 @@ class Util:
         elif re.match(r"\d{2}/\d{2}/\d{2} \d{2}h\d{2}$", formatted_date):
             return datetime.datetime.strptime(formatted_date, '%d/%m/%y %Hh%M')
         else:
-            log.warning("date parsing not possible for value '" + formatted_date + "' set to None")
-            return None
+            raise DateParsingError("Could not parse date", formatted_date)
 
     @staticmethod
     def html_parse(html_text: str) -> str:
@@ -99,7 +101,7 @@ class Object:
             self.json_dict = json_dict
 
         def __call__(self, converter: Callable[[Any], R], *path: str, default: Union[MissingType, R] = _missing,
-                     strict: bool = True) -> Optional[R]:
+                     strict: bool = True) -> R:
             """
 
             Parameters
@@ -141,7 +143,7 @@ class Object:
 
             return json_value
 
-    def __init__(self, json_dict):
+    def __init__(self, json_dict: dict) -> None:
         self._resolver: Object._Resolver = self._Resolver(json_dict)
 
 
@@ -161,12 +163,51 @@ class Subject(Object):
 
     __slots__ = ['id', 'name', 'groups']
 
-    def __init__(self, parsed_json):
+    def __init__(self, parsed_json: dict) -> None:
         super().__init__(parsed_json)
 
-        self.id = self._resolver(str, "N")
-        self.name = self._resolver(str, "L")
-        self.groups = self._resolver(bool, "estServiceGroupe", default=False)
+        self.id: str = self._resolver(str, "N")
+        self.name: str = self._resolver(str, "L")
+        self.groups: bool = self._resolver(bool, "estServiceGroupe", default=False)
+
+        del self._resolver
+
+
+class Absence(Object):
+    """
+    Represents an absence with a given period. You shouldn't have to create this class manually.
+
+    Attributes
+    ----------
+    id : str
+        the id of the absence (used internally)
+    from_date : datetime.datetime
+        starting time of the absence
+    to_date : datetime.datetime
+        end of the absence
+    justified : bool
+        is the absence justified
+    hours : str
+        the number of hours missed
+    days : int
+        the number of days missed
+    reasons: List[str]
+        The reason(s) for the absence
+    """
+
+    __slots__ = ['id', 'from_date', 'to_date', 'justified',
+                 'hours' 'days', 'reasons']
+
+    def __init__(self, json_dict: dict) -> None:
+        super().__init__(json_dict)
+
+        self.id: str = self._resolver(str, "N")
+        self.from_date: datetime.datetime = self._resolver(Util.datetime_parse, "dateDebut", "V")
+        self.to_date: datetime.datetime = self._resolver(Util.datetime_parse, "dateFin", "V")
+        self.justified: bool = self._resolver(bool, "justifie", default=False)
+        self.hours: Optional[str] = self._resolver(str, "NbrHeures", strict=False)
+        self.days: int = self._resolver(int, "NbrJours", default=0)
+        self.reasons: List[str] = self._resolver(lambda l: [i["L"] for i in l], "listeMotifs", "V", default=[])
 
         del self._resolver
 
@@ -190,7 +231,7 @@ class Period(Object):
     __slots__ = ['_client', 'id', 'name', 'start', 'end']
     instances: Set[Any] = set()
 
-    def __init__(self, client, json_dict):
+    def __init__(self, client: _ClientBase, json_dict: dict) -> None:
         super().__init__(json_dict)
 
         self.__class__.instances.add(self)
@@ -204,34 +245,34 @@ class Period(Object):
         del self._resolver
 
     @property
-    def grades(self):
+    def grades(self) -> List["Grade"]:
         """Get grades from the period."""
-        json_data = {'periode': {'N': self.id, 'L': self.name}}
+        json_data = {'Periode': {'N': self.id, 'L': self.name}}
         response = self._client.post('DernieresNotes', 198, json_data)
         log.debug(response)
         grades = response['donneesSec']['donnees']['listeDevoirs']['V']
         return [Grade(g) for g in grades]
 
     @property
-    def averages(self):
+    def averages(self) -> List["Average"]:
         """Get averages from the period."""
 
-        json_data = {'periode': {'N': self.id, 'L': self.name}}
+        json_data = {'Periode': {'N': self.id, 'L': self.name}}
         response = self._client.post('DernieresNotes', 198, json_data)
         crs = response['donneesSec']['donnees']['listeServices']['V']
         return [Average(c) for c in crs]
 
     @property
-    def overall_average(self):
+    def overall_average(self) -> float:
         """Get overall average from the period. If the period average is not provided by pronote, then it's calculated.
         Calculation may not be the same as the actual average. (max difference 0.01)"""
-        json_data = {'periode': {'N': self.id, 'L': self.name}}
+        json_data = {'Periode': {'N': self.id, 'L': self.name}}
         response = self._client.post('DernieresNotes', 198, json_data)
         average = response['donneesSec']['donnees'].get('moyGenerale')
         if average:
             average = average['V']
         elif response['donneesSec']['donnees']['listeServices']['V']:
-            a = 0
+            a: float = 0
             total = 0
             services = response['donneesSec']['donnees']['listeServices']['V']
             for s in services:
@@ -252,12 +293,32 @@ class Period(Object):
         return average
 
     @property
-    def evaluations(self):
+    def evaluations(self) -> List["Evaluation"]:
         json_data = {'periode': {'N': self.id, 'L': self.name, 'G': 2}}
         response = self._client.post('DernieresEvaluations', 201, json_data)
         evaluations = response['donneesSec']['donnees']['listeEvaluations']['V']
         return [Evaluation(e) for e in evaluations]
 
+    @property
+    def absences(self) -> List[Absence]:
+        """
+        Gets all absences in a given period.
+
+        Returns
+        -------
+        List[Absence]
+            All the absences of the period
+        """
+        json_data = {
+            'periode': {'N': self.id, 'L': self.name, 'G': 2},
+            'DateDebut': {'_T': 7, 'V': self.start.strftime("%d/%m/%Y %H:%M:%S")},
+            'DateFin': {'_T': 7, 'V': self.end.strftime("%d/%m/%Y %H:%M:%S")}
+        }
+
+        response = self._client.post('PagePresence', 19, json_data)
+        absences = response['donneesSec']['donnees']['listeAbsences']['V']
+        return [Absence(a) for a in absences if a["G"] == 13]
+
 
 class Average(Object):
     """
@@ -283,7 +344,7 @@ class Average(Object):
 
     __slots__ = ['student', 'out_of', 'default_out_of', 'class_average', 'min', 'max', 'subject']
 
-    def __init__(self, json_dict):
+    def __init__(self, json_dict: dict) -> None:
         super().__init__(json_dict)
 
         self.student: str = self._resolver(Util.grade_parse, "moyEleve", "V")
@@ -331,7 +392,7 @@ class Grade(Object):
     __slots__ = ['id', 'grade', 'out_of', 'default_out_of', 'date', 'subject',
                  'period', 'average', 'max', 'min', 'coefficient', 'comment']
 
-    def __init__(self, json_dict):
+    def __init__(self, json_dict: dict) -> None:
         super().__init__(json_dict)
 
         self.id: str = self._resolver(str, "N")
@@ -350,6 +411,104 @@ class Grade(Object):
         del self._resolver
 
 
+class File(Object):
+    """
+    Represents a file uploaded to pronote.
+
+    Attributes
+    ----------
+    name : str
+        Name of the file.
+    id : str
+        id of the file (used internally and for url)
+    url : str
+        url of the file
+    """
+
+    __slots__ = ['name', 'id', '_client', 'url', '_data']
+
+    def __init__(self, client: _ClientBase, json_dict: dict) -> None:
+        super().__init__(json_dict)
+
+        self._client = client
+
+        self.name: str = self._resolver(str, 'L')
+        self.id: str = self._resolver(str, 'N')
+
+        padd = Padding.pad(json.dumps({'N': self.id, 'G': 1}).replace(' ', '').encode(), 16)
+        magic_stuff = client.communication.encryption.aes_encrypt(padd).hex()
+
+        self.url = f"{client.communication.root_site}/FichiersExternes/{magic_stuff}/" + \
+                   quote(self.name, safe='~()*!.\'') + f"?Session={client.attributes['h']}"
+
+        self._data = None
+
+        del self._resolver
+
+    def save(self, file_name: str = None) -> None:
+        """
+        Saves the file on to local storage.
+
+        Parameters
+        ----------
+        file_name : str
+            file name
+        """
+        response = self._client.communication.session.get(self.url)
+        if not file_name:
+            file_name = self.name
+        if response.status_code == 200:
+            with open(file_name, 'wb') as handle:
+                for block in response.iter_content(1024):
+                    handle.write(block)
+        else:
+            raise FileNotFoundError("The file was not found on pronote. The url may be badly formed.")
+
+    @property
+    def data(self) -> bytes:
+        """Gets the raw file data."""
+        if self._data:
+            return self._data
+        response = self._client.communication.session.get(self.url)
+        return response.content
+
+
+class LessonContent(Object):
+    """
+    Represents the content of a lesson. You shouldn't have to create this class manually.
+
+    Attributes
+    ----------
+    title : Optional[str]
+        title of the lesson content
+    description : Optional[str]
+        description of the lesson content
+    category : Optional[str]
+        category of the lesson content
+    files : tuple
+        files attached on lesson content
+    """
+
+    __slots__ = ['title', 'description', '_files', '_client']
+
+    def __init__(self, client: _ClientBase, json_dict: dict) -> None:
+        super().__init__(json_dict)
+
+        self._client = client
+
+        self.title: Optional[str] = self._resolver(str, 'L', strict=False)
+        self.description: Optional[str] = self._resolver(Util.html_parse, "descriptif", "V", strict=False)
+        self.category: Optional[str] = self._resolver(str, "categorie", "V", "L", strict=False)
+        self._files: Tuple[Any, ...] = self._resolver(tuple, "ListePieceJointe", "V")
+
+        del self._resolver
+
+    @property
+    def files(self) -> List[File]:
+        """Get all the attached files from the lesson"""
+        return [File(self._client, jsn) for jsn in self._files]
+
+
 class Lesson(Object):
     """
     Represents a lesson with a given time. You shouldn't have to create this class manually.
@@ -362,8 +521,12 @@ class Lesson(Object):
         the subject that the lesson is from
     teacher_name : Optional[str]
         name of the teacher
+    teacher_names : Optional[List[str]]
+        name of the teachers
     classroom : Optional[str]
         name of the classroom
+    classrooms : Optional[List[str]]
+        name of the classrooms
     canceled : bool
         if the lesson is canceled
     status : Optional[str]
@@ -378,6 +541,8 @@ class Lesson(Object):
         end of the lesson
     group_name : Optional[str]
         Name of the group.
+    group_names : Optional[List[str]]
+        Name of the groups.
     exempted : bool
         Specifies if the student's presence is exempt.
     virtual_classrooms : List[str]
@@ -388,22 +553,16 @@ class Lesson(Object):
         is marked as detention
     """
 
-    __slots__ = ['id', 'subject', 'teacher_name', 'classroom', 'start',
-                 'canceled', 'status', 'background_color', 'detention',
-                 'end', 'outing', 'group_name', 'exempted',
+    __slots__ = ['id', 'subject', 'teacher_name', 'teacher_names',
+                 'classroom', 'classrooms', 'start', 'canceled',
+                 'status', 'background_color', 'detention', 'end',
+                 'outing', 'group_name', 'group_names', 'exempted',
                  '_client', '_content', 'virtual_classrooms', 'num']
 
-    transformers = {
-        16: ('subject', Subject),
-        3: ('teacher_name', _get_l),
-        17: ('classroom', _get_l),
-        2: ('group_name', _get_l)
-    }
-
-    def __init__(self, client, json_dict):
+    def __init__(self, client: _ClientBase, json_dict: dict) -> None:
         super().__init__(json_dict)
         self._client = client
-        self._content = None
+        self._content: Optional[LessonContent] = None
 
         self.id: str = self._resolver(str, "N")
         self.canceled: bool = self._resolver(bool, "estAnnule", default=False)
@@ -416,6 +575,7 @@ class Lesson(Object):
                                                             default=[])
         self.num: int = self._resolver(int, "P", default=0)
         self.detention: bool = self._resolver(bool, "estRetenue", default=False)
+        self.test: bool = self._resolver(bool, "cahierDeTextes", "V", "estDevoir", default=False)
 
         # get correct ending time
         # Pronote gives us the place where the hour should be in a week, when we modulo that with the amount of
@@ -433,33 +593,44 @@ class Lesson(Object):
         self.end: datetime.datetime = self.start.replace(hour=end_time.hour, minute=end_time.minute)
 
         # get additional information about the lesson
-        self.teacher_name: Optional[str] = None
-        self.classroom: Optional[str] = None
-        self.group_name: Optional[str] = None
+        self.teacher_names: Optional[List[str]] = []
+        self.classrooms: Optional[List[str]] = []
+        self.group_names: Optional[List[str]] = []
         self.subject: Optional[Subject] = None
 
         if 'ListeContenus' not in json_dict:
             raise ParsingError("Error while parsing for lesson details", json_dict, ("ListeContenus", "V"))
 
         for d in json_dict['ListeContenus']['V']:
-            try:
-                self.__setattr__(self.__class__.transformers[d['G']][0], self.__class__.transformers[d['G']][1](d))
-            except KeyError:
-                pass
+            if 'G' not in d:
+                continue
+            elif d['G'] == 16:
+                self.subject = Subject(d)
+            elif d['G'] == 3:
+                self.teacher_names.append(d['L'])
+            elif d['G'] == 17:
+                self.classrooms.append(d['L'])
+            elif d['G'] == 2:
+                self.group_names.append(d['L'])
+
+        # All values joined together to prevent breaking changes
+        self.teacher_name: Optional[str] = ', '.join(self.teacher_names) if self.teacher_names else None
+        self.classroom: Optional[str] = ', '.join(self.classrooms) if self.classrooms else None
+        self.group_name: Optional[str] = ', '.join(self.group_names) if self.group_names else None
 
         del self._resolver
 
     @property
-    def normal(self):
+    def normal(self) -> bool:
         if self.detention is None and self.outing is None:
             return True
         return False
 
     @property
-    def content(self):
+    def content(self) -> Optional[LessonContent]:
         """
         Gets content of the lesson. May be None if there is no description.
-        
+
         Notes
         -----
         This property is very inefficient and will send a request to pronote, so don't use it often.
@@ -480,99 +651,6 @@ class Lesson(Object):
         return self._content
 
 
-class LessonContent(Object):
-    """
-    Represents the content of a lesson. You shouldn't have to create this class manually.
-
-    Attributes
-    ----------
-    title : str
-        title of the lesson content
-    description : str
-        description of the lesson content
-    """
-
-    __slots__ = ['title', 'description', '_files', '_client']
-
-    def __init__(self, client, json_dict):
-        super().__init__(json_dict)
-
-        self._client = client
-
-        self.title: str = self._resolver(str, 'L')
-        self.description: str = self._resolver(Util.html_parse, "descriptif", "V")
-        self._files: Tuple[dict] = self._resolver(tuple, "ListePieceJointe", "V")
-
-        del self._resolver
-
-    @property
-    def files(self):
-        """Get all the attached files from the lesson"""
-        return [File(self._client, jsn) for jsn in self._files]
-
-
-class File(Object):
-    """
-    Represents a file uploaded to pronote.
-
-    Attributes
-    ----------
-    name : str
-        Name of the file.
-    id : str
-        id of the file (used internally and for url)
-    url : str
-        url of the file
-    """
-
-    __slots__ = ['name', 'id', '_client', 'url', '_data']
-
-    def __init__(self, client, json_dict):
-        super().__init__(json_dict)
-
-        self._client = client
-
-        self.name: str = self._resolver(str, 'L')
-        self.id: str = self._resolver(str, 'N')
-
-        padd = Padding.pad(json.dumps({'N': self.id, 'G': 1}).replace(' ', '').encode(), 16)
-        magic_stuff = client.communication.encryption.aes_encrypt(padd).hex()
-
-        self.url = f"{client.communication.root_site}/FichiersExternes/{magic_stuff}/" + \
-                   quote(self.name, safe='~()*!.\'') + f"?Session={client.attributes['h']}"
-
-        self._data = None
-
-        del self._resolver
-
-    def save(self, file_name=None):
-        """
-        Saves the file on to local storage.
-
-        Parameters
-        ----------
-        file_name : str
-            file name
-        """
-        response = self._client.communication.session.get(self.url)
-        if not file_name:
-            file_name = self.name
-        if response.status_code == 200:
-            with open(file_name, 'wb') as handle:
-                for block in response.iter_content(1024):
-                    handle.write(block)
-        else:
-            raise FileNotFoundError("The file was not found on pronote. The url may be badly formed.")
-
-    @property
-    def data(self):
-        """Gets the raw file data."""
-        if self._data:
-            return self._data
-        response = self._client.communication.session.get(self.url)
-        return response.content
-
-
 class Homework(Object):
     """
     Represents a homework. You shouldn't have to create this class manually.
@@ -586,7 +664,7 @@ class Homework(Object):
     description : str
         the description of the homework
     background_color : str
-        the background color of the homework 
+        the background color of the homework
     done : bool
         if the homework is marked done
     date : datetime.date
@@ -595,7 +673,7 @@ class Homework(Object):
 
     __slots__ = ['id', 'subject', 'description', 'done', 'background_color', '_client', 'date', '_files']
 
-    def __init__(self, client, json_dict):
+    def __init__(self, client: _ClientBase, json_dict: dict) -> None:
         super().__init__(json_dict)
 
         self._client = client
@@ -610,7 +688,7 @@ class Homework(Object):
 
         del self._resolver
 
-    def set_done(self, status: bool):
+    def set_done(self, status: bool) -> None:
         """
         Sets the status of the homework.
 
@@ -626,9 +704,9 @@ class Homework(Object):
         self.done = status
 
     @property
-    def files(self):
+    def files(self) -> List[File]:
         """Get all the files attached to the homework"""
-        return [File(self._client, jsn) for jsn in self._files]
+        return [File(self._client, jsn) for jsn in self._files]  # type: ignore
 
 
 class Information(Object):
@@ -662,7 +740,7 @@ class Information(Object):
     __slots__ = ['id', 'title', 'author', 'read', 'creation_date', 'start_date', 'end_date', 'category',
                  'survey', 'anonymous_response', '_raw_content', '_client']
 
-    def __init__(self, client, json_dict):
+    def __init__(self, client: _ClientBase, json_dict: dict) -> None:
         super().__init__(json_dict)
 
         self._client = client
@@ -682,10 +760,10 @@ class Information(Object):
         del self._resolver
 
     @property
-    def content(self):
+    def content(self) -> str:
         return Util.html_parse(self._raw_content[0]['texte']['V'])
 
-    def mark_as_read(self, status: bool):
+    def mark_as_read(self, status: bool) -> None:
         data = {
             "listeActualites": [
                 {
@@ -725,7 +803,7 @@ class Message(Object):
 
     __slots__ = ['id', 'author', 'recipients', 'seen', 'date', '_client', '_listePM']
 
-    def __init__(self, client, json_dict):
+    def __init__(self, client: _ClientBase, json_dict: dict) -> None:
         super().__init__(json_dict)
         self._client = client
         self._listePM = json_dict['listePossessionsMessages']['V']
@@ -740,7 +818,7 @@ class Message(Object):
         del self._resolver
 
     @property
-    def content(self):
+    def content(self) -> Optional[str]:
         """Returns the content of the message"""
         data = {'message': {'N': self.id},
                 'marquerCommeLu': False,
@@ -768,11 +846,13 @@ class ClientInfo:
       Raw json defining the resource
     """
 
-    __slots__ = ['id', 'raw_resource']
+    __slots__ = ['id', 'raw_resource', '_client', '__cache']
 
-    def __init__(self, json_):
+    def __init__(self, client: _ClientBase, json_: dict) -> None:
         self.id: str = json_['N']
         self.raw_resource: dict = json_
+        self._client = client
+        self.__cache: Optional[dict] = None
 
     @property
     def name(self) -> str:
@@ -805,6 +885,55 @@ class ClientInfo:
         """
         return self.raw_resource.get('Etablissement', {'V': {'L': ""}})['V']['L']
 
+    def _cache(self) -> dict:
+        if self.__cache is None:
+            # this does not have all the protection _ClientBase.post provides,
+            # but we need to manually add the resource id
+            self.__cache = self._client.communication.post(
+                "PageInfosPerso",
+                {"_Signature_": {"onglet": 49, "ressource": {"N": self.id, "G": 4}}}
+            )["donneesSec"]["donnees"]["Informations"]
+
+        return self.__cache
+
+    @property
+    def address(self) -> tuple[str, str, str, str, str, str, str, str]:
+        """
+        Address of the client
+
+        Returns
+        -------
+        A tuple of 8 elements:
+            - 4 lines of address info
+            - postal code
+            - city
+            - province
+            - country
+        """
+        c = self._cache()
+        return c["adresse1"], c["adresse2"], c["adresse3"], c["adresse4"], c["codePostal"], c["ville"], c["province"], \
+               c["pays"]
+
+    @property
+    def email(self) -> str:
+        return self._cache()["eMail"]
+
+    @property
+    def phone(self) -> str:
+        """
+        Phone of the client
+
+        Returns
+        -------
+        Phone in the format +[country-code][phone-number]
+        """
+        c = self._cache()
+        return "+" + c["indicatifTel"] + c["telephonePortable"]
+
+    @property
+    def ine_number(self) -> str:
+        return self._cache()["numeroINE"]
+
 
 class Acquisition(Object):
     """
@@ -837,7 +966,7 @@ class Acquisition(Object):
         "order", "level", "id", "abbreviation", "coefficient", "domain", "domain_id", "name", "name_id", "pillar",
         "pillar_id", "pillar_prefix"]
 
-    def __init__(self, json_dict):
+    def __init__(self, json_dict: dict) -> None:
         super().__init__(json_dict)
 
         self.id: str = self._resolver(str, "N")
@@ -879,7 +1008,7 @@ class Evaluation(Object):
         "name", "name", "id", "domain", "teacher", "coefficient", "description", "subject", "paliers", "acquisitions",
         "date"]
 
-    def __init__(self, json_dict):
+    def __init__(self, json_dict: dict) -> None:
         super().__init__(json_dict)
         self.name: str = self._resolver(str, "L")
         self.id: str = self._resolver(str, "N")
@@ -899,7 +1028,7 @@ class Evaluation(Object):
 class Identity(Object):
     """
     Represents an Identity of a person
-    
+
     Attributes
     ----------
     postal_code : str
@@ -920,7 +1049,7 @@ class Identity(Object):
     __slots__ = ["postal_code", "date_of_birth", "email", "last_name", "country", "mobile_number", "landline_number",
                  "other_phone_number", "city", "place_of_birth", "first_names", "address", "formatted_address"]
 
-    def __init__(self, json_dict):
+    def __init__(self, json_dict: dict) -> None:
         super().__init__(json_dict)
 
         self.postal_code: str = self._resolver(str, "CP")
@@ -942,7 +1071,8 @@ class Identity(Object):
             self.address.append(option)
             i += 1
         self.formatted_address: str = ','.join([*self.address, self.postal_code, self.city, self.country])
-        self.first_names: List[str] = [json_dict.get("prenom"), json_dict.get("prenom2"), json_dict.get("prenom3")]
+        self.first_names: List[str] = [json_dict.get("prenom", ""), json_dict.get("prenom2", ""),
+                                       json_dict.get("prenom3", "")]
 
         del self._resolver
 
@@ -970,7 +1100,7 @@ class Guardian(Object):
                  "preferred_responsible_contact", "accomodates_kid", "relatives_link", "responsibility_level",
                  "financially_responsible", "full_name", "is_legal"]
 
-    def __init__(self, json_dict):
+    def __init__(self, json_dict: dict) -> None:
         super().__init__(json_dict)
 
         self.accepteInfosProf: bool = self._resolver(bool, "accepteInfosProf")
@@ -1010,7 +1140,7 @@ class Student(Object):
     __slots__ = ["_client", "full_name", "id", "enrollment_date", "date_of_birth", "projects", "last_name",
                  "first_names", "sex", "options", "_cache"]
 
-    def __init__(self, client, json_dict):
+    def __init__(self, client: _ClientBase, json_dict: dict) -> None:
         super().__init__(json_dict)
 
         self.full_name: str = self._resolver(str, "L")
@@ -1024,7 +1154,7 @@ class Student(Object):
         self.sex: str = self._resolver(str, "sexe")
 
         self._client = client
-        self._cache = None
+        self._cache: Optional[dict] = None
 
         self.options = []
         i = 1
@@ -1042,7 +1172,7 @@ class Student(Object):
         """
         Identity of this student
         """
-        if not self._cache:
+        if self._cache is None:
             self._cache = self._client.post("FicheEleve", 105,
                                             {"Eleve": {"N": self.id}, "AvecEleve": True, "AvecResponsables": True})
         return Identity(self._cache["donneesSec"]["donnees"]["Identite"])
@@ -1052,7 +1182,7 @@ class Student(Object):
         """
         List of responsible persons (parents).
         """
-        if not self._cache:
+        if self._cache is None:
             self._cache = self._client.post("FicheEleve", 105,
                                             {"Eleve": {"N": self.id}, "AvecEleve": True, "AvecResponsables": True})
         return [Guardian(j) for j in self._cache["donneesSec"]["donnees"]["Responsables"]["V"]]
@@ -1072,7 +1202,7 @@ class StudentClass(Object):
     """
     __slots__ = ["name", "id", "responsible", "grade", "_client"]
 
-    def __init__(self, client, json_dict):
+    def __init__(self, client: _ClientBase, json_dict: dict) -> None:
         super().__init__(json_dict)
 
         self.name: str = self._resolver(str, "L")
@@ -1089,3 +1219,113 @@ class StudentClass(Object):
         r = self._client.post("ListeRessources", 105,
                               {"classe": {"N": self.id, "G": 1}, "periode": {"N": period.id, "G": 1}})
         return [Student(self._client, j) for j in r["donneesSec"]["donnees"]["listeRessources"]["V"]]
+
+
+class Menu(Object):
+    """
+    Represents the menu of a meal
+
+    Attributes
+    ----------
+    id : str
+    self.name: Optional[str]
+    date : datetime.date
+        the date of the menu
+    is_lunch : bool
+        the menu is a lunch menu
+    is_dinner : bool
+        the menu is a dinner menu
+    first_meal : Optional[List[Food]]
+        food list of first meal
+    main_meal : Optional[List[Food]]
+        food list of main meal
+    side_meal : Optional[List[Food]]
+        food list of side meal
+    other_meal : Optional[List[Food]]
+        food list of other meal
+    cheese : Optional[List[Food]]
+        food list of cheese
+    dessert : Optional[List[Food]]
+        food list of dessert
+    """
+
+    class Food(Object):
+        """
+        Represents food of a menu
+
+        Attributes
+        ----------
+        id : str
+        name : str
+        labels : List[FoodLabel]
+        """
+
+        class FoodLabel(Object):
+            """
+            Represents the label of a food
+
+            Attributes
+            ----------
+            id : str
+            name: str
+            color: str
+            """
+
+            __slots__ = ["name", "id", "color", "_client"]
+
+            def __init__(self, client: _ClientBase, json_dict: dict) -> None:
+                super().__init__(json_dict)
+
+                self.id: str = self._resolver(str, "N")
+                self.name: str = self._resolver(str, "L")
+                self.color: str = self._resolver(str, 'couleur')
+
+                self._client = client
+
+                del self._resolver
+
+        __slots__ = ["name", "id", "labels", "_client"]
+
+        def __init__(self, client: _ClientBase, json_dict: dict) -> None:
+            super().__init__(json_dict)
+
+            self.id: str = self._resolver(str, "N")
+            self.name: str = self._resolver(str, "L")
+            self.labels: List[Menu.Food.FoodLabel] = self._resolver(
+                lambda labels: [self.FoodLabel(client, label) for label in labels],
+                "listeLabelsAlimentaires", "V"
+            )
+
+            self._client = client
+
+            del self._resolver
+
+    __slots__ = ["name", "id", "date", "is_lunch", "is_dinner", "first_meal",
+                 "main_meal", "side_meal", "other_meal", "cheese", "dessert", "_client"]
+
+    def __init__(self, client: _ClientBase, json_dict: dict) -> None:
+        super().__init__(json_dict)
+
+        self.id: str = self._resolver(str, "N")
+        self.name: Optional[str] = self._resolver(str, "L", strict=False)
+        self.date: datetime.date = self._resolver(Util.date_parse, 'Date', 'V')
+
+        self.is_lunch: bool = self._resolver(int, 'G') == 0
+        self.is_dinner: bool = self._resolver(int, 'G') == 1
+
+        def init_food(d: dict) -> List[Menu.Food]:
+            return [self.Food(client, x) for x in d["ListeAliments"]["V"]]
+
+        d_dict = {str(meal['G']): meal for meal in json_dict["ListePlats"]["V"]}
+        super().__init__(d_dict)
+
+        self.first_meal: Optional[List[Menu.Food]] = self._resolver(init_food, "0", strict=False)
+        self.main_meal: Optional[List[Menu.Food]] = self._resolver(init_food, "1", strict=False)
+        self.side_meal: Optional[List[Menu.Food]] = self._resolver(init_food, "2", strict=False)
+        self.other_meal: Optional[List[Menu.Food]] = self._resolver(init_food, "3", strict=False)
+        self.cheese: Optional[List[Menu.Food]] = self._resolver(init_food, "5", strict=False)
+        self.dessert: Optional[List[Menu.Food]] = self._resolver(init_food, "4", strict=False)
+
+        self._client = client
+
+        del self._resolver
