@@ -1,4 +1,4 @@
-from typing import List, Tuple
+from typing import Tuple
 
 
 class PronoteAPIError(Exception):
@@ -24,6 +24,7 @@ class ExpiredObject(PronoteAPIError):
 
 class ChildNotFound(PronoteAPIError):
     """Child with this name was not found."""
+    pass
 
 
 class DataError(Exception):
@@ -39,3 +40,15 @@ class ParsingError(DataError):
         super().__init__(message)
         self.json_dict = json_dict
         self.path = path
+
+
+class ICalExportError(PronoteAPIError):
+    """Error while exporting ICal. Pronote did not return token"""
+    pass
+
+
+class DateParsingError(PronoteAPIError):
+    """Bad date string"""
+    def __init__(self, message: str, date_string: str):
+        super().__init__(message)
+        self.date_string = date_string
